﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.SolutionCaching.RealVectorEncoding {
  [Item("RealVectorSolutionCachingAnalyzer", "An operator that records all RealVector solutions every generation for GAs.")]
  [StorableClass]
  public class RealVectorSolutionCachingAnalyzer : SingleSuccessorOperator, IAnalyzer {
    private const string ResultsParameterName = "Results";

    #region IAnalyzer Members
    public bool EnabledByDefault {
      get { return false; }
    }
    #endregion

    #region Parameter properties
    public ILookupParameter<ResultCollection> ResultsParameter {
      get { return (ILookupParameter<ResultCollection>)Parameters[ResultsParameterName]; }
    }
    public ILookupParameter<IntValue> GenerationsParameter {
      get { return (ILookupParameter<IntValue>)Parameters["Generations"]; }
    }
    public ScopeTreeLookupParameter<RealVector> SolutionParameter {
      get { return (ScopeTreeLookupParameter<RealVector>)Parameters["Solution"]; }
    }
    #endregion

    #region Properties
    public ResultCollection Results {
      get { return ResultsParameter.ActualValue; }
    }
    #endregion

    [StorableConstructor]
    private RealVectorSolutionCachingAnalyzer(bool deserializing) : base(deserializing) { }

    private RealVectorSolutionCachingAnalyzer(RealVectorSolutionCachingAnalyzer original, Cloner cloner)
      : base(original, cloner) {
    }

    public RealVectorSolutionCachingAnalyzer()
      : base() {
      Parameters.Add(new LookupParameter<ResultCollection>(ResultsParameterName, "The results collection where the analysis values should be stored."));
      Parameters.Add(new LookupParameter<IntValue>("Generations", "Nr of generations."));
      Parameters.Add(new ScopeTreeLookupParameter<RealVector>("Solution", "The solutions that should be stored.", "Point", 1));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new RealVectorSolutionCachingAnalyzer(this, cloner);
    }

    public override IOperation Apply() {
      ItemArray<RealVector> solutions = SolutionParameter.ActualValue;
      RealVectorSolutionCache solDict;

      if (!Results.ContainsKey("SolutionCache")) {
        solDict = new RealVectorSolutionCache();
        Results.Add(new Result("SolutionCache", solDict));
      } else {
        solDict = (RealVectorSolutionCache)Results["SolutionCache"].Value;
      }

      foreach (var sol in solutions) {
        RealVectorSolutionInformation info = new RealVectorSolutionInformation();
        info.Generation = GenerationsParameter.ActualValue.Value;
        info.ProducedBy = ProducedBy.CrossoverAndMutation;

        solDict.Add((RealVector)sol.Clone(new Cloner()), info);
      }
      return base.Apply();
    }
  }
}
